/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

/** @module vertx-auth-oauth2-js/access_token */
var utils = require('vertx-js/util/utils');
var User = require('vertx-auth-common-js/user');
var Buffer = require('vertx-js/buffer');
var OAuth2Response = require('vertx-auth-oauth2-js/o_auth2_response');

var io = Packages.io;
var JsonObject = io.vertx.core.json.JsonObject;
var JAccessToken = Java.type('io.vertx.ext.auth.oauth2.AccessToken');

/**
 AccessToken extension to the User interface

 @class
*/
var AccessToken = function(j_val) {

  var j_accessToken = j_val;
  var that = this;
  User.call(this, j_val);

  var __super_expired = this.expired;
  var __super_accessToken = this.accessToken;
  var __super_refreshToken = this.refreshToken;
  var __super_idToken = this.idToken;
  var __super_opaqueAccessToken = this.opaqueAccessToken;
  var __super_opaqueRefreshToken = this.opaqueRefreshToken;
  var __super_opaqueIdToken = this.opaqueIdToken;
  var __super_tokenType = this.tokenType;
  var __super_setTrustJWT = this.setTrustJWT;
  var __super_refresh = this.refresh;
  var __super_revoke = this.revoke;
  var __super_logout = this.logout;
  var __super_introspect = this.introspect;
  var __super_introspect = this.introspect;
  var __super_userInfo = this.userInfo;
  var __super_fetch = this.fetch;
  var __super_fetch = this.fetch;
  /**
   Check if the access token is expired or not.

   @public

   @return {boolean}
   */
  this.expired =  function() {
    var __args = arguments;
    if (__args.length === 0) {
      return j_accessToken["expired()"]() ;
    } else if (typeof __super_expired != 'undefined') {
      return __super_expired.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   The Access Token if present parsed as a JsonObject

   @public

   @return {Object} JSON
   */
  this.accessToken =  function() {
    var __args = arguments;
    if (__args.length === 0) {
      if (that.cachedaccessToken == null) {
        that.cachedaccessToken = utils.convReturnJson(j_accessToken["accessToken()"]());
      }
      return that.cachedaccessToken;
    } else if (typeof __super_accessToken != 'undefined') {
      return __super_accessToken.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   The Refresh Token if present parsed as a JsonObject

   @public

   @return {Object} JSON
   */
  this.refreshToken =  function() {
    var __args = arguments;
    if (__args.length === 0) {
      if (that.cachedrefreshToken == null) {
        that.cachedrefreshToken = utils.convReturnJson(j_accessToken["refreshToken()"]());
      }
      return that.cachedrefreshToken;
    } else if (typeof __super_refreshToken != 'undefined') {
      return __super_refreshToken.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   The Id Token if present parsed as a JsonObject

   @public

   @return {Object} JSON
   */
  this.idToken =  function() {
    var __args = arguments;
    if (__args.length === 0) {
      if (that.cachedidToken == null) {
        that.cachedidToken = utils.convReturnJson(j_accessToken["idToken()"]());
      }
      return that.cachedidToken;
    } else if (typeof __super_idToken != 'undefined') {
      return __super_idToken.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   The RAW String if available for the Access Token

   @public

   @return {string} String
   */
  this.opaqueAccessToken =  function() {
    var __args = arguments;
    if (__args.length === 0) {
      return j_accessToken["opaqueAccessToken()"]() ;
    } else if (typeof __super_opaqueAccessToken != 'undefined') {
      return __super_opaqueAccessToken.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   The RAW String if available for the Refresh Token

   @public

   @return {string} String
   */
  this.opaqueRefreshToken =  function() {
    var __args = arguments;
    if (__args.length === 0) {
      return j_accessToken["opaqueRefreshToken()"]() ;
    } else if (typeof __super_opaqueRefreshToken != 'undefined') {
      return __super_opaqueRefreshToken.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   The RAW String if available for the Id Token

   @public

   @return {string} String
   */
  this.opaqueIdToken =  function() {
    var __args = arguments;
    if (__args.length === 0) {
      return j_accessToken["opaqueIdToken()"]() ;
    } else if (typeof __super_opaqueIdToken != 'undefined') {
      return __super_opaqueIdToken.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**

   @public

   @return {string}
   */
  this.tokenType =  function() {
    var __args = arguments;
    if (__args.length === 0) {
      return j_accessToken["tokenType()"]() ;
    } else if (typeof __super_tokenType != 'undefined') {
      return __super_tokenType.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**

   @public
   @param trust {boolean} 
   @return {AccessToken}
   */
  this.setTrustJWT =  function(trust) {
    var __args = arguments;
    if (__args.length === 1 && typeof __args[0] ==='boolean') {
      j_accessToken["setTrustJWT(boolean)"](__args[0]) ;
      return that;
    } else if (typeof __super_setTrustJWT != 'undefined') {
      return __super_setTrustJWT.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   Refresh the access token

   @public
   @param callback {function} - The callback function returning the results. 
   @return {AccessToken}
   */
  this.refresh =  function(callback) {
    var __args = arguments;
    if (__args.length === 1 && typeof __args[0] === 'function') {
      j_accessToken["refresh(io.vertx.core.Handler)"](function(ar) {
        if (ar.succeeded()) {
          __args[0](null, null);
        } else {
          __args[0](null, ar.cause());
        }
      }) ;
      return that;
    } else if (typeof __super_refresh != 'undefined') {
      return __super_refresh.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   Revoke access or refresh token

   @public
   @param token_type {string} - A String containing the type of token to revoke. Should be either "access_token" or "refresh_token". 
   @param callback {function} - The callback function returning the results. 
   @return {AccessToken}
   */
  this.revoke =  function(token_type, callback) {
    var __args = arguments;
    if (__args.length === 2 && typeof __args[0] === 'string' && typeof __args[1] === 'function') {
      j_accessToken["revoke(java.lang.String,io.vertx.core.Handler)"](__args[0], function(ar) {
        if (ar.succeeded()) {
          __args[1](null, null);
        } else {
          __args[1](null, ar.cause());
        }
      }) ;
      return that;
    } else if (typeof __super_revoke != 'undefined') {
      return __super_revoke.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   Revoke refresh token and calls the logout endpoint. This is a openid-connect extension and might not be
   available on all providers.

   @public
   @param callback {function} - The callback function returning the results. 
   @return {AccessToken}
   */
  this.logout =  function(callback) {
    var __args = arguments;
    if (__args.length === 1 && typeof __args[0] === 'function') {
      j_accessToken["logout(io.vertx.core.Handler)"](function(ar) {
        if (ar.succeeded()) {
          __args[0](null, null);
        } else {
          __args[0](null, ar.cause());
        }
      }) ;
      return that;
    } else if (typeof __super_logout != 'undefined') {
      return __super_logout.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   Introspect access token. This is an OAuth2 extension that allow to verify if an access token is still valid.

   @public
   @param tokenType {string} - A String containing the type of token to revoke. Should be either "access_token" or "refresh_token". 
   @param callback {function} - The callback function returning the results. 
   @return {AccessToken}
   */
  this.introspect =  function() {
    var __args = arguments;
    if (__args.length === 1 && typeof __args[0] === 'function') {
      j_accessToken["introspect(io.vertx.core.Handler)"](function(ar) {
        if (ar.succeeded()) {
          __args[0](null, null);
        } else {
          __args[0](null, ar.cause());
        }
      }) ;
      return that;
    } else if (__args.length === 2 && typeof __args[0] === 'string' && typeof __args[1] === 'function') {
      j_accessToken["introspect(java.lang.String,io.vertx.core.Handler)"](__args[0], function(ar) {
        if (ar.succeeded()) {
          __args[1](null, null);
        } else {
          __args[1](null, ar.cause());
        }
      }) ;
      return that;
    } else if (typeof __super_introspect != 'undefined') {
      return __super_introspect.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   Load the user info as per OIDC spec.

   @public
   @param callback {function} - The callback function returning the results. 
   @return {AccessToken}
   */
  this.userInfo =  function(callback) {
    var __args = arguments;
    if (__args.length === 1 && typeof __args[0] === 'function') {
      j_accessToken["userInfo(io.vertx.core.Handler)"](function(ar) {
        if (ar.succeeded()) {
          __args[0](utils.convReturnJson(ar.result()), null);
        } else {
          __args[0](null, ar.cause());
        }
      }) ;
      return that;
    } else if (typeof __super_userInfo != 'undefined') {
      return __super_userInfo.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   Fetches a JSON resource using this Access Token.

   @public
   @param method {Object} - the HTTP method to user. 
   @param resource {string} - the resource to fetch. 
   @param headers {Object} - extra headers to pass to the request. 
   @param payload {Buffer} - payload to send to the server. 
   @param callback {function} - The callback function returning the results. 
   @return {AccessToken}
   */
  this.fetch =  function() {
    var __args = arguments;
    if (__args.length === 2 && typeof __args[0] === 'string' && typeof __args[1] === 'function') {
      j_accessToken["fetch(java.lang.String,io.vertx.core.Handler)"](__args[0], function(ar) {
        if (ar.succeeded()) {
          __args[1](utils.convReturnVertxGen(OAuth2Response, ar.result()), null);
        } else {
          __args[1](null, ar.cause());
        }
      }) ;
      return that;
    } else if (__args.length === 5 && typeof __args[0] === 'string' && typeof __args[1] === 'string' && (typeof __args[2] === 'object' && __args[2] != null) && typeof __args[3] === 'object' && __args[3]._jdel && typeof __args[4] === 'function') {
      j_accessToken["fetch(io.vertx.core.http.HttpMethod,java.lang.String,io.vertx.core.json.JsonObject,io.vertx.core.buffer.Buffer,io.vertx.core.Handler)"](io.vertx.core.http.HttpMethod.valueOf(__args[0]), __args[1], utils.convParamJsonObject(__args[2]), __args[3]._jdel, function(ar) {
        if (ar.succeeded()) {
          __args[4](utils.convReturnVertxGen(OAuth2Response, ar.result()), null);
        } else {
          __args[4](null, ar.cause());
        }
      }) ;
      return that;
    } else if (typeof __super_fetch != 'undefined') {
      return __super_fetch.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  // A reference to the underlying Java delegate
  // NOTE! This is an internal API and must not be used in user code.
  // If you rely on this property your code is likely to break if we change it / remove it without warning.
  this._jdel = j_accessToken;
};

AccessToken._jclass = utils.getJavaClass("io.vertx.ext.auth.oauth2.AccessToken");
AccessToken._jtype = {accept: function(obj) {
    return AccessToken._jclass.isInstance(obj._jdel);
  },wrap: function(jdel) {
    var obj = Object.create(AccessToken.prototype, {});
    AccessToken.apply(obj, arguments);
    return obj;
  },
  unwrap: function(obj) {
    return obj._jdel;
  }
};
AccessToken._create = function(jdel) {var obj = Object.create(AccessToken.prototype, {});
  AccessToken.apply(obj, arguments);
  return obj;
}
module.exports = AccessToken;